/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */
/******************************************************************************
 * tigerwww.c - main program file                                             *
 *****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "vector.h"
#include "units.h"
#define DEF_WIDTH 512
#define DEF_HEIGHT 256

XFontStruct *vlargeFont, *vsmallFont;
extern Widget toplevel, draww;


static MapStruct *ms = NULL;

void DrawVectorMap(Pixmap p, Dimension width, Dimension height, double lat,
	      double lon, double wi, double hi, int projection)
{
    map_init();

    ms->proj = projection;

    ms->CtrLonD = lon;
    ms->CtrLatD = lat;

    ms->minWidthD = wi;
    ms->HeightD = hi;

    ms->WidthP = width;
    ms->HeightP = height;

    /* Draw Image */
    DrawMap(ms, p);
}

decode(int x, int y, double *lat, double *lon)
{
    long latu, lonu, xm, xp;


    latu = ms->CtrLatU - PIX_TO_MU(ms, (y - (ms->HeightP / 2)));

    if (abs(latu) < DEG90_LU) {

        xm = PIX_TO_MU(ms, x) - ms->HalfWidU;                                                              xp = cos((double) (abs(latu) >> 9) * PI / (DEG180_LU >> 9)) * (1 << XPU_SHIFT);
                                                                                                           lonu = ms->CtrLonU + MU_TO_LONU(xm, xp, ms->GenShift);

        if (abs(lonu) < DEG360_LU) {
            if (lonu > DEG180_LU)
                lonu -= DEG360_LU;
            else if (lonu < -DEG180_LU)
                lonu += DEG360_LU;
            *lat = LU_TO_DEG(latu);
            *lon = LU_TO_DEG(lonu);
        }
    }
}

void project(double lat, double lon, Position * x, Position * y)
{
    int ix, iy;

    Project(ms, (long) (lat * 1000000.0), (long) (lon * 1000000.0), &ix, &iy);
    *x = ix;
    *y = iy;
}

long vscale()
{
    return ms->scale;
}

/***** INITIALIZE MAP PARAMETERS STRUCTURE *****/
MapStruct *map_init()
{
    if (!ms) {
	ms = (MapStruct *) calloc(1, sizeof(MapStruct));
	/* Allocate memory for dynamic parts */
	ms->datasets = (Config *) calloc(MAXDATASETS, sizeof(Config));
	ms->layers = (Config *) calloc(MAXLAYERS, sizeof(Config));
	ms->features = (Config *) calloc(MAXFEATURES, sizeof(Config));

	vsmallFont = XLoadQueryFont(XtDisplay(draww),
			     "-adobe-times-medium-r-*-*-*-80-*-*-*-*-*-*"
			//   "-adobe-courier-medium-r-*-*-*-80-*-*-*-*-*-*"
			//   "-*-clean-medium-r-*-*-*-80-*-*-*-*-*-*"
			//   "-*-helvetica-medium-r-*-*-*-80-*-*-*-*-*-*"
	    );

	vlargeFont = XLoadQueryFont(XtDisplay(toplevel),
			    "-*-helvetica-bold-r-*-*-*-100-*-*-*-*-*-*");
    }
    memset(ms->datasets, 0, sizeof(Config) * MAXDATASETS);
    memset(ms->layers, 0, sizeof(Config) * MAXLAYERS);
    memset(ms->features, 0, sizeof(Config) * MAXFEATURES);

    /* Initialize fields */
    ms->ndirs = 0;
    ms->nlyrs = 0;
    ms->nfeats = 0;
    ms->GridSpace = DEF_GRID;	/* Assume Default Grid Spacing */
    ms->debug = DEF_DEBUG;	/* No Debugging */
    ms->EdgeSD = DEF_S;		/* Assume Continental U.S. */
    ms->EdgeND = DEF_N;
    ms->EdgeWD = DEF_W;
    ms->EdgeED = DEF_E;
    ms->CtrLonD = (DEF_W + DEF_E) / 2;
    ms->CtrLatD = (DEF_N + DEF_S) / 2;
    ms->minWidthD = DEF_E - DEF_W;
    ms->maxWidthD = DEF_E - DEF_W;
    ms->HeightD = DEF_N - DEF_S;
    ms->WidthP = DEF_WIDTH;
    ms->HeightP = DEF_HEIGHT;
    ms->CtrLatU = 0;
    ms->CtrLonU = 0;
    ms->HalfWidU = DEG180_LU;
    ms->BoundsU.SLat = DEG_TO_LU(DEF_S);
    ms->BoundsU.NLat = DEG_TO_LU(DEF_N);
    ms->BoundsU.ELon = DEG_TO_LU(DEF_W);
    ms->BoundsU.WLon = DEG_TO_LU(DEF_E);
    ms->muPerPix = 1;

    return (ms);
}
